/* _NVRM_COPYRIGHT_BEGIN_
 *
 * Copyright 2015 by NVIDIA Corporation.  All rights reserved.  All
 * information contained herein is proprietary and confidential to NVIDIA
 * Corporation.  Any use, reproduction, or disclosure without the written
 * permission of NVIDIA Corporation is prohibited.
 *
 * _NVRM_COPYRIGHT_END_
 */

#ifndef _NV_PCI_TABLE_H_
#define _NV_PCI_TABLE_H_

#include <linux/pci.h>

extern struct pci_device_id nv_pci_table[];

#endif /* _NV_PCI_TABLE_H_ */
