/* _NVRM_COPYRIGHT_BEGIN_
 *
 * Copyright 2015 by NVIDIA Corporation.  All rights reserved.  All
 * information contained herein is proprietary and confidential to NVIDIA
 * Corporation.  Any use, reproduction, or disclosure without the written
 * permission of NVIDIA Corporation is prohibited.
 *
 * _NVRM_COPYRIGHT_END_
 */

#ifndef _RMP2PDEFINES_H_
#define _RMP2PDEFINES_H_

#define NVRM_P2P_PAGESIZE_SMALL_4K   (4 << 10)
#define NVRM_P2P_PAGESIZE_BIG_64K    (64 << 10)
#define NVRM_P2P_PAGESIZE_BIG_128K   (128 << 10)

#endif
